@@ -88,6 +88,11 @@ public:
 
 	void LoadSongList(struct mpdclient &c);
 
+protected:
+	/* virtual methods from class FileListPage */
+	bool HandleEnter(struct mpdclient &c) override;
+
+public:
 	/* virtual methods from class Page */
 	void Update(struct mpdclient &c, unsigned events) noexcept override;
 	bool OnCommand(struct mpdclient &c, Command cmd) override;
@@ -102,9 +107,28 @@ SongListPage::Update(struct mpdclient &c, unsigned events) noexcept
 	}
 }
 
+class ArtistBrowserPage;
+
+class LibraryTagListPage final : public TagListPage {
+	ArtistBrowserPage &library_page;
+
+public:
+	LibraryTagListPage(ScreenManager &_screen,
+			   ArtistBrowserPage &_library_page,
+			   Page *_parent,
+			   const enum mpd_tag_type _tag,
+			   const char *_all_text,
+			   WINDOW *_w, Size size) noexcept
+		:TagListPage(_screen, _parent, _tag, _all_text, _w, size),
+		 library_page(_library_page) {}
+
+protected:
+	bool HandleEnter(struct mpdclient &c) override;
+};
+
 class ArtistBrowserPage final : public ProxyPage {
-	std::list<TagListPage> tag_list_pages;
-	std::list<TagListPage>::iterator current_tag_list_page;
+	std::list<LibraryTagListPage> tag_list_pages;
+	std::list<LibraryTagListPage>::iterator current_tag_list_page;
 
 	SongListPage song_list_page;
 
@@ -117,7 +141,7 @@ public:
 
 		bool first = true;
 		for (const auto &tag : options.library_page_tags) {
-			tag_list_pages.emplace_back(_screen,
+			tag_list_pages.emplace_back(_screen, *this,
 						    first ? nullptr : this,
 						    tag,
 						    first ? nullptr : _("All"),
@@ -126,6 +150,8 @@ public:
 		}
 	}
 
+	void EnterTag(struct mpdclient &c, TagFilter &&filter);
+
 private:
 	void OpenTagPage(struct mpdclient &c,
 			 TagFilter &&filter) noexcept;
@@ -197,18 +223,20 @@ SongListPage::GetTitle(char *str, size_t size) const noexcept
 	return MakePageTitle(str, size, _("Songs"), filter);
 }
 
+bool
+SongListPage::HandleEnter(struct mpdclient &c)
+{
+	if (lw.GetCursorIndex() == 0 && parent != nullptr)
+		/* handle ".." */
+		return parent->OnCommand(c, Command::GO_PARENT_DIRECTORY);
+
+	return FileListPage::HandleEnter(c);
+}
+
 bool
 SongListPage::OnCommand(struct mpdclient &c, Command cmd)
 {
 	switch(cmd) {
-	case Command::PLAY:
-		if (lw.GetCursorIndex() == 0 && parent != nullptr)
-			/* handle ".." */
-			return parent->OnCommand(c, Command::GO_PARENT_DIRECTORY);
-
-		break;
-
-		/* continue and update... */
 	case Command::SCREEN_UPDATE:
 		LoadSongList(c);
 		return false;
@@ -244,6 +272,43 @@ ArtistBrowserPage::Update(struct mpdclient &c, unsigned events) noexcept
 	ProxyPage::Update(c, events);
 }
 
+inline void
+ArtistBrowserPage::EnterTag(struct mpdclient &c, TagFilter &&filter)
+{
+	assert(current_tag_list_page != tag_list_pages.end());
+
+	++current_tag_list_page;
+
+	if (current_tag_list_page != tag_list_pages.end()) {
+		while (true) {
+			OpenTagPage(c,  std::move(filter));
+			if (current_tag_list_page->HasMultipleValues())
+				break;
+
+			/* skip tags which have just
+			   one value */
+			filter = current_tag_list_page->GetFilter();
+			++current_tag_list_page;
+			if (current_tag_list_page == tag_list_pages.end()) {
+				OpenSongList(c, std::move(filter));
+				break;
+			}
+		}
+	} else
+		OpenSongList(c, std::move(filter));
+}
+
+bool
+LibraryTagListPage::HandleEnter(struct mpdclient &c)
+{
+	auto new_filter = MakeCursorFilter();
+	if (new_filter.empty())
+		return TagListPage::HandleEnter(c);
+
+	library_page.EnterTag(c, std::move(new_filter));
+	return true;
+}
+
 bool
 ArtistBrowserPage::OnCommand(struct mpdclient &c, Command cmd)
 {
@@ -251,23 +316,6 @@ ArtistBrowserPage::OnCommand(struct mpdclient &c, Command cmd)
 		return true;
 
 	switch (cmd) {
-	case Command::PLAY:
-		if (current_tag_list_page != tag_list_pages.end()) {
-			auto filter = current_tag_list_page->MakeCursorFilter();
-			if (filter.empty())
-				break;
-
-			++current_tag_list_page;
-
-			if (current_tag_list_page != tag_list_pages.end())
-				OpenTagPage(c,  std::move(filter));
-			else
-				OpenSongList(c, std::move(filter));
-			return true;
-		}
-
-		break;
-
 	case Command::GO_ROOT_DIRECTORY:
 		if (GetCurrentPage() != &tag_list_pages.front()) {
 			current_tag_list_page = tag_list_pages.begin();
@@ -278,10 +326,13 @@ ArtistBrowserPage::OnCommand(struct mpdclient &c, Command cmd)
 		break;
 
 	case Command::GO_PARENT_DIRECTORY:
-		if (current_tag_list_page != tag_list_pages.begin()) {
+		while (current_tag_list_page != tag_list_pages.begin()) {
 			--current_tag_list_page;
 			SetCurrentPage(c, &*current_tag_list_page);
-			return true;
+			if (current_tag_list_page->HasMultipleValues())
+				return true;
+
+			/* skip tags which have just one value */
 		}
 
 		break;
