@@ -89,7 +89,7 @@ ListCursor::ScrollTo(unsigned n) noexcept
 {
 	int new_start = start;
 
-	if ((unsigned) options.scroll_offset * 2 >= GetHeight())
+	if (options.scroll_offset * 2 >= GetHeight())
 		// Center if the offset is more than half the screen
 		new_start = n - GetHeight() / 2;
 	else {
@@ -131,12 +131,18 @@ ListCursor::MoveCursor(unsigned n) noexcept
 void
 ListCursor::FetchCursor() noexcept
 {
+	/* clamp the scroll-offset setting to slightly less than half
+	   of the screen height */
+	const unsigned scroll_offset = options.scroll_offset * 2 < GetHeight()
+		? options.scroll_offset
+		: std::max(GetHeight() / 2, 1U) - 1;
+
 	if (start > 0 &&
-	    selected < start + options.scroll_offset)
-		MoveCursor(start + options.scroll_offset);
+	    selected < start + scroll_offset)
+		MoveCursor(start + scroll_offset);
 	else if (start + GetHeight() < length &&
-		 selected > start + GetHeight() - 1 - options.scroll_offset)
-		MoveCursor(start + GetHeight() - 1 - options.scroll_offset);
+		 selected > start + GetHeight() - 1 - scroll_offset)
+		MoveCursor(start + GetHeight() - 1 - scroll_offset);
 }
 
 ListWindowRange
@@ -182,7 +188,7 @@ ListCursor::MoveCursorTop() noexcept
 	if (start == 0)
 		MoveCursor(start);
 	else
-		if ((unsigned) options.scroll_offset * 2 >= GetHeight())
+		if (options.scroll_offset * 2 >= GetHeight())
 			MoveCursor(start + GetHeight() / 2);
 		else
 			MoveCursor(start + options.scroll_offset);
@@ -201,7 +207,7 @@ void
 ListCursor::MoveCursorBottom() noexcept
 {
 	if (length >= GetHeight())
-		if ((unsigned) options.scroll_offset * 2 >= GetHeight())
+		if (options.scroll_offset * 2 >= GetHeight())
 			MoveCursor(start + GetHeight() / 2);
 		else
 			if (start + GetHeight() == length)
