@@ -123,11 +123,8 @@ Instance::UpdateClient() noexcept
 }
 
 void
-Instance::OnReconnectTimer(const boost::system::error_code &error) noexcept
+Instance::OnReconnectTimer() noexcept
 {
-	if (error)
-		return;
-
 	assert(client.IsDead());
 
 	screen_status_printf(_("Connecting to %s"),
@@ -198,11 +195,8 @@ mpdclient_idle_callback(gcc_unused unsigned events) noexcept
 }
 
 void
-Instance::OnUpdateTimer(const boost::system::error_code &error) noexcept
+Instance::OnUpdateTimer() noexcept
 {
-	if (error)
-		return;
-
 	assert(pending_update_timer);
 	pending_update_timer = false;
 
@@ -226,17 +220,17 @@ void end_input_event() noexcept
 }
 
 bool
-do_input_event(boost::asio::io_service &io_service, Command cmd) noexcept
+do_input_event(EventLoop &event_loop, Command cmd) noexcept
 {
 	if (cmd == Command::QUIT) {
-		io_service.stop();
+		event_loop.Break();
 		return false;
 	}
 
 	try {
 		screen->OnCommand(*mpd, global_instance->GetSeek(), cmd);
 	} catch (...) {
-		screen_status_message(GetFullMessage(std::current_exception()).c_str());
+		screen_status_error(std::current_exception());
 		return true;
 	}
 
@@ -255,7 +249,7 @@ do_mouse_event(Point p, mmask_t bstate) noexcept
 	try {
 		screen->OnMouse(*mpd, global_instance->GetSeek(), p, bstate);
 	} catch (...) {
-		screen_status_message(GetFullMessage(std::current_exception()).c_str());
+		screen_status_error(std::current_exception());
 	}
 }
 
@@ -267,11 +261,8 @@ do_mouse_event(Point p, mmask_t bstate) noexcept
  * message every 10 seconds.
  */
 void
-Instance::OnCheckKeyBindings(const boost::system::error_code &error) noexcept
+Instance::OnCheckKeyBindings() noexcept
 {
-	if (error)
-		return;
-
 	char buf[256];
 
 	if (GetGlobalKeyBindings().Check(buf, sizeof(buf)))
