@@ -30,12 +30,12 @@
 
 #include <string.h>
 
-StatusBar::StatusBar(boost::asio::io_service &io_service,
+StatusBar::StatusBar(EventLoop &event_loop,
 		     Point p, unsigned width) noexcept
 	:window(p, {width, 1u}),
-	 message_timer(io_service)
+	 message_timer(event_loop, BIND_THIS_METHOD(OnMessageTimer))
 #ifndef NCMPC_MINI
-	, hscroll(io_service, window.w, options.scroll_sep.c_str())
+	, hscroll(event_loop, window.w, options.scroll_sep.c_str())
 #endif
 {
 	leaveok(window.w, false);
@@ -58,7 +58,7 @@ StatusBar::~StatusBar() noexcept
 void
 StatusBar::ClearMessage() noexcept
 {
-	message_timer.cancel();
+	message_timer.Cancel();
 	message.clear();
 
 	Paint();
@@ -97,14 +97,10 @@ FormatCurrentSongTime(char *buffer, size_t size,
 		return;
 
 	const unsigned total_time = mpd_status_get_total_time(&status);
-	if (total_time == 0)
-		return;
 
 	unsigned elapsed_time = seek.IsSeeking(mpd_status_get_song_id(&status))
 		? seek.GetTime()
 		: mpd_status_get_elapsed_time(&status);
-	if (elapsed_time == 0)
-		return;
 
 	char elapsed_string[32], duration_string[32];
 
@@ -116,6 +112,9 @@ FormatCurrentSongTime(char *buffer, size_t size,
 		break;
 
 	case CurrentTimeDisplay::REMAINING:
+		if (total_time == 0)
+			return;
+
 		elapsed_time = elapsed_time < total_time
 			? total_time - elapsed_time
 			: 0;
@@ -126,6 +125,12 @@ FormatCurrentSongTime(char *buffer, size_t size,
 	format_duration_short(elapsed_string,
 			      sizeof(elapsed_string),
 			      elapsed_time);
+
+	if (total_time == 0) {
+		snprintf(buffer, size, " [%s]", elapsed_string);
+		return;
+	}
+
 	format_duration_short(duration_string,
 			      sizeof(duration_string),
 			      total_time);
@@ -193,14 +198,16 @@ StatusBar::Update(const struct mpd_status *status,
 
 		/* scroll if the song name is to long */
 #ifndef NCMPC_MINI
-		center_width = StringWidthMB(center_text.c_str());
-		if (options.scroll && width > 3 &&
-		    center_width > (unsigned)width) {
-			hscroll.Set(left_width, 0, width, center_text.c_str(),
-				    Style::STATUS);
-		} else {
-			if (options.scroll)
+		if (options.scroll) {
+			const unsigned center_width =
+				StringWidthMB(center_text.c_str());
+			if (width > 3 && center_width > (unsigned)width) {
+				hscroll.Set(left_width, 0, width,
+					    center_text.c_str(),
+					    Style::STATUS);
+			} else {
 				hscroll.Clear();
+			}
 		}
 #endif
 	} else {
@@ -284,9 +291,5 @@ StatusBar::SetMessage(const char *msg) noexcept
 	Paint();
 	doupdate();
 
-	boost::system::error_code error;
-	message_timer.expires_from_now(options.status_message_time,
-				       error);
-	message_timer.async_wait(std::bind(&StatusBar::OnMessageTimer, this,
-					   std::placeholders::_1));
+	message_timer.Schedule(options.status_message_time);
 }
