@@ -166,6 +166,16 @@ TagListPage::Update(struct mpdclient &c, unsigned events) noexcept
 	}
 }
 
+bool
+TagListPage::HandleEnter(struct mpdclient &c)
+{
+	if (lw.GetCursorIndex() == 0 && parent != nullptr)
+		/* handle ".." */
+		return parent->OnCommand(c, Command::GO_PARENT_DIRECTORY);
+
+	return false;
+}
+
 /* add_query - Add all songs satisfying specified criteria.
    _artist is actually only used in the ALBUM case to distinguish albums with
    the same name from different artists. */
@@ -178,8 +188,10 @@ add_query(struct mpdclient *c, const TagFilter &filter,
 		return;
 
 	const char *text = value;
-	if (value == nullptr)
-		value = filter.empty() ? "?" : filter.front().second.c_str();
+	if (text == nullptr)
+		/* adding the special "[All]" entry: show the name of
+		   the previous level in the filter */
+		text = filter.empty() ? "?" : filter.front().second.c_str();
 
 	screen_status_printf(_("Adding \'%s\' to queue"),
 			     Utf8ToLocale(text).c_str());
@@ -195,32 +207,41 @@ add_query(struct mpdclient *c, const TagFilter &filter,
 	c->FinishCommand();
 }
 
+bool
+TagListPage::HandleSelect(struct mpdclient &c)
+{
+	bool result = false;
+
+	for (unsigned i : lw.GetRange()) {
+		if (parent != nullptr) {
+			if (i == 0)
+				continue;
+
+			--i;
+		}
+
+		add_query(&c, filter, tag,
+			  i < values.size()
+			  ? values[i].c_str() : nullptr);
+		result = true;
+	}
+
+	return result;
+}
+
+
 bool
 TagListPage::OnCommand(struct mpdclient &c, Command cmd)
 {
 	switch(cmd) {
 	case Command::PLAY:
-		if (lw.GetCursorIndex() == 0 && parent != nullptr)
-			/* handle ".." */
-			return parent->OnCommand(c, Command::GO_PARENT_DIRECTORY);
-
-		break;
+		return HandleEnter(c);
 
 	case Command::SELECT:
 	case Command::ADD:
-		for (unsigned i : lw.GetRange()) {
-			if (parent != nullptr) {
-				if (i == 0)
-					continue;
-
-				--i;
-			}
-
-			add_query(&c, filter, tag,
-				  i < values.size()
-				  ? values[i].c_str() : nullptr);
+		if (HandleSelect(c))
 			cmd = Command::LIST_NEXT; /* continue and select next item... */
-		}
+
 		break;
 
 		/* continue and update... */
@@ -252,3 +273,31 @@ TagListPage::OnCommand(struct mpdclient &c, Command cmd)
 
 	return false;
 }
+
+#ifdef HAVE_GETMOUSE
+
+bool
+TagListPage::OnMouse(struct mpdclient &c, Point p,
+		      mmask_t bstate)
+{
+	unsigned prev_selected = lw.GetCursorIndex();
+
+	if (ListPage::OnMouse(c, p, bstate))
+		return true;
+
+	lw.SetCursorFromOrigin(p.y);
+
+	if( bstate & BUTTON1_CLICKED ) {
+		if (prev_selected == lw.GetCursorIndex())
+			HandleEnter(c);
+	} else if (bstate & BUTTON3_CLICKED) {
+		if (prev_selected == lw.GetCursorIndex())
+			HandleSelect(c);
+	}
+
+	SetDirty();
+
+	return true;
+}
+
+#endif
