@@ -29,103 +29,94 @@
 #include "aconnect.hxx"
 #include "net/AsyncResolveConnect.hxx"
 #include "net/AsyncHandler.hxx"
+#include "net/UniqueSocketDescriptor.hxx"
+#include "system/Error.hxx"
+#include "event/SocketEvent.hxx"
 
 #include <mpd/client.h>
 #include <mpd/async.h>
 
-#include <boost/asio/generic/stream_protocol.hpp>
+#include <cerrno>
+#include <cstdio>
+#include <cstring>
 
 struct AsyncMpdConnect final : AsyncConnectHandler {
 	AsyncMpdConnectHandler &handler;
 
 	AsyncResolveConnect rconnect;
 
-	boost::asio::generic::stream_protocol::socket socket;
+	SocketEvent socket;
 
-	char buffer[256];
-
-	explicit AsyncMpdConnect(boost::asio::io_service &io_service,
+	explicit AsyncMpdConnect(EventLoop &event_loop,
 				 AsyncMpdConnectHandler &_handler) noexcept
 		:handler(_handler),
-		 rconnect(io_service, *this), socket(io_service) {}
+		 rconnect(event_loop, *this),
+		 socket(event_loop, BIND_THIS_METHOD(OnReceive)) {}
 
-	void OnReceive(const boost::system::error_code &error,
-		       std::size_t bytes_transferred) noexcept;
+	~AsyncMpdConnect() noexcept {
+		socket.Close();
+	}
+
+	void OnReceive(unsigned events) noexcept;
 
 	/* virtual methods from AsyncConnectHandler */
-	void OnConnect(boost::asio::generic::stream_protocol::socket socket) override;
-	void OnConnectError(const char *message) override;
+	void OnConnect(UniqueSocketDescriptor fd) noexcept override;
+	void OnConnectError(std::exception_ptr e) noexcept override;
 };
 
 void
-AsyncMpdConnect::OnReceive(const boost::system::error_code &error,
-			   std::size_t bytes_transferred) noexcept
-{
-	if (error) {
-		if (error == boost::asio::error::operation_aborted)
-			/* this object has already been deleted; bail out
-			   quickly without touching anything */
-			return;
+AsyncMpdConnect::OnReceive(unsigned) noexcept
+try {
+	char buffer[256];
+	ssize_t nbytes = socket.GetSocket().Read(buffer, sizeof(buffer));
 
-		snprintf(buffer, sizeof(buffer),
-			 "Failed to receive from MPD: %s",
-			 error.message().c_str());
-		handler.OnAsyncMpdConnectError(buffer);
-		delete this;
-		return;
-	}
+	if (nbytes < 0)
+		throw MakeErrno("Failed to receive from MPD");
 
-	buffer[bytes_transferred] = 0;
+	buffer[nbytes] = 0;
 
-	/* the dup() is necessary because Boost 1.62 doesn't have the
-	   release() method yet */
-	struct mpd_async *async = mpd_async_new(dup(socket.native_handle()));
-	if (async == nullptr) {
-		handler.OnAsyncMpdConnectError("Out of memory");
-		delete this;
-		return;
-	}
+	struct mpd_async *async = mpd_async_new(socket.ReleaseSocket().Get());
+	if (async == nullptr)
+		throw std::bad_alloc{};
 
 	struct mpd_connection *c = mpd_connection_new_async(async, buffer);
 	if (c == nullptr) {
 		mpd_async_free(async);
-		handler.OnAsyncMpdConnectError("Out of memory");
-		delete this;
-		return;
+		throw std::bad_alloc{};
 	}
 
 	handler.OnAsyncMpdConnect(c);
 	delete this;
-}
-
-void
-AsyncMpdConnect::OnConnect(boost::asio::generic::stream_protocol::socket _socket)
-{
-	socket = std::move(_socket);
-	socket.async_receive(boost::asio::buffer(buffer, sizeof(buffer) - 1),
-			     std::bind(&AsyncMpdConnect::OnReceive, this,
-				       std::placeholders::_1,
-				       std::placeholders::_2));
-}
-
-void
-AsyncMpdConnect::OnConnectError(const char *message)
-{
-	handler.OnAsyncMpdConnectError(message);
+} catch (...) {
+	handler.OnAsyncMpdConnectError(std::current_exception());
 	delete this;
 }
 
 void
-aconnect_start(boost::asio::io_service &io_service,
+AsyncMpdConnect::OnConnect(UniqueSocketDescriptor fd) noexcept
+{
+	socket.Open(fd.Release());
+	socket.ScheduleRead();
+}
+
+void
+AsyncMpdConnect::OnConnectError(std::exception_ptr e) noexcept
+{
+	handler.OnAsyncMpdConnectError(std::move(e));
+	delete this;
+}
+
+void
+aconnect_start(EventLoop &event_loop,
 	       AsyncMpdConnect **acp,
 	       const char *host, unsigned port,
 	       AsyncMpdConnectHandler &handler)
 {
-	auto *ac = new AsyncMpdConnect(io_service, handler);
+	auto *ac = new AsyncMpdConnect(event_loop, handler);
 
 	*acp = ac;
 
-	ac->rconnect.Start(io_service, host, port);
+	ac->rconnect.Start(host, port);
 }
 
 void
