@@ -28,28 +28,59 @@
 
 #include "AsyncConnect.hxx"
 #include "AsyncHandler.hxx"
+#include "net/UniqueSocketDescriptor.hxx"
+#include "net/SocketAddress.hxx"
+#include "system/Error.hxx"
+
+#include <exception>
+
+static UniqueSocketDescriptor
+Connect(const SocketAddress address)
+{
+	UniqueSocketDescriptor fd;
+	if (!fd.CreateNonBlock(address.GetFamily(), SOCK_STREAM, 0))
+		throw MakeErrno("Failed to create socket");
+
+	if (!fd.Connect(address) && errno != EINPROGRESS)
+		throw MakeErrno("Failed to connect");
+
+	return fd;
+}
+
+bool
+AsyncConnect::Start(const SocketAddress address) noexcept
+{
+	assert(!event.IsDefined());
+
+	try {
+		WaitConnected(::Connect(address));
+		return true;
+	} catch (...) {
+		handler.OnConnectError(std::current_exception());
+		return false;
+	}
+}
 
 void
-AsyncConnect::OnConnected(const boost::system::error_code &error) noexcept
+AsyncConnect::WaitConnected(UniqueSocketDescriptor fd) noexcept
 {
-	if (error) {
-		if (error == boost::asio::error::operation_aborted)
-			/* this object has already been deleted; bail out
-			   quickly without touching anything */
-			return;
+	assert(!event.IsDefined());
 
-		socket.close();
-		handler.OnConnectError(error.message().c_str());
-		return;
+	event.Open(fd.Release());
+	event.ScheduleWrite();
+}
+
+void
+AsyncConnect::OnSocketReady(unsigned events) noexcept
+{
+	if (SocketEvent::ERROR == 0 || events & SocketEvent::ERROR) {
+		int s_err = event.GetSocket().GetError();
+		if (s_err != 0) {
+			event.Close();
+			handler.OnConnectError(std::make_exception_ptr(MakeErrno(s_err, "Failed to connect")));
+			return;
+		}
 	}
 
-	handler.OnConnect(std::move(socket));
-}
-
-void
-AsyncConnect::Start(const boost::asio::ip::tcp::endpoint &endpoint) noexcept
-{
-	socket.async_connect(endpoint,
-			     std::bind(&AsyncConnect::OnConnected, this,
-				       std::placeholders::_1));
+	handler.OnConnect(UniqueSocketDescriptor(event.ReleaseSocket()));
 }
