@@ -20,6 +20,7 @@
 #include "screen_status.hxx"
 #include "screen.hxx"
 #include "ncmpc.hxx"
+#include "util/Exception.hxx"
 
 #include <stdarg.h>
 
@@ -39,3 +40,9 @@ screen_status_printf(const char *format, ...) noexcept
 	va_end(ap);
 	screen_status_message(msg);
 }
+
+void
+screen_status_error(std::exception_ptr e) noexcept
+{
+	screen_status_message(GetFullMessage(std::move(e)).c_str());
+}
