@@ -20,30 +20,16 @@
 #include "Instance.hxx"
 
 void
-Instance::OnSigwinch()
+Instance::OnSigwinch() noexcept
 {
 	endwin();
 	refresh();
 	screen_manager.OnResize();
 }
 
-void
-Instance::AsyncWaitSigwinch()
-{
-	sigwinch.async_wait([this](const auto &error, int){
-			if (error)
-				return;
-
-			this->OnSigwinch();
-			this->AsyncWaitSigwinch();
-		});
-}
-
 void
 Instance::InitSignals()
 {
-	AsyncWaitSigwinch();
-
 	/* ignore SIGPIPE */
 
 	struct sigaction act;
